indexing

   description: 
   "Describes memory related security."
   
   usage: "Inherit from STDC_SECURITY_ACCESSOR."
   
   author: "Berend de Boer"
   date: "$Date: 2001/12/27 $"
   revision: "$Revision: #1 $"

class 

   STDC_SECURITY_MEMORY
   
   
inherit
   
   STDC_SECURITY_ASPECT

   
creation
   
   make
   
   
feature -- creation
   
   make is
         -- allow everything
      do
         max_allocated_memory := Max_Int
         max_single_allocation := Max_Int
      end
   

feature -- parameters
   
   allocated_memory: INTEGER

   max_allocated_memory: INTEGER
         -- maximum allowed memory allocation in bytes
   
   max_single_allocation: INTEGER
         -- maximum allowed single memory block in bytes

   
feature {STDC_SECURITY_ACCESSOR} -- set parameters
   
   set_max_allocated_memory (value: INTEGER) is
      do
         max_allocated_memory := value
         check_max_allocation
      ensure
         max_allocated_memory >= allocated_memory
      end
   
   set_max_single_allocation (value: INTEGER) is
      do
         max_single_allocation := Value
      end
   
   
feature {STDC_SECURITY_ACCESSOR} -- count resource usage
   
   decrease_memory_allocation (amount: INTEGER) is
      do
         allocated_memory := allocated_memory - amount
      end
   
   increase_memory_allocation (amount: INTEGER) is
      do
         allocated_memory := allocated_memory + amount
         check_max_allocation
      end


feature -- checks
   
   is_allowed_allocation (a_size: INTEGER): BOOLEAN is
      do
         Result := a_size <= max_single_allocation
      end      
   
   check_allocation (a_size: INTEGER) is
      do
         if not is_allowed_allocation (a_size) then
            raise_security_error ("Cannot allocate " + a_size.out + 
                                  " memory. Maximum allowed is " +
                                  max_single_allocation.out + ".%N")
         end
      end
   
   check_max_allocation is
      do
         if allocated_memory > max_allocated_memory then
            raise_security_error ("Maximum allowed memory allocation exceeded.%N")
         end
      end
   
   
invariant
   
   positive_allocated_memory: allocated_memory >= 0
   positive_max_allocated_memory: max_allocated_memory >= 0
   positive_max_single_allocation: max_single_allocation >= 0


end -- class STDC_SECURITY_MEMORY
